@@ -298,7 +298,7 @@ int popcnt(unsigned long val)
 
 int max_node, numnodes;
 
-void get_node_list()
+int get_node_list(void)
 {
         int a, got_nodes = 0;
         long free_node_sizes;
@@ -307,9 +307,12 @@ void get_node_list()
         node_to_use = (int *)malloc(numnodes * sizeof(int));
         max_node = numa_max_node();
         for (a = 0; a <= max_node; a++) {
-                if(numa_node_size(a, &free_node_sizes) != -1)
+                if (numa_node_size(a, &free_node_sizes) > 0)
                         node_to_use[got_nodes++] = a;
         }
+        if(got_nodes != numnodes)
+                return -1;
+        return 0;
 }
 
 void test(enum test type)
@@ -390,6 +393,8 @@ void test(enum test type)
 		printf("current interleave node %d\n", numa_get_interleave_node());
 	}
 
+	numa_bitmask_free(nodes);
+
 	numa_set_interleave_mask(numa_no_nodes_ptr);
 
 	nodes = numa_allocate_nodemask();
@@ -436,7 +441,7 @@ void test(enum test type)
 		numa_set_localalloc();
 		memtest("local allocation", numa_alloc(msize));
 
-		numa_set_preferred((node_to_use[i]+1) % numnodes );
+		numa_set_preferred(node_to_use[(i + 1) % numnodes]);
 		memtest("setting wrong preferred node", numa_alloc(msize));
 		numa_set_preferred(node_to_use[i]);
 		memtest("setting correct preferred node", numa_alloc(msize));
@@ -444,7 +449,7 @@ void test(enum test type)
 		if (!delim[0])
 			printf("\n\n\n");
 	}
-
+	numa_bitmask_free(nodes);
 	/* numa_run_on_node_mask is not tested */
 }
 
@@ -512,7 +517,11 @@ int main(int ac, char **av)
 		if (!force)
 			exit(1);
 	}
-	get_node_list();
+	if(get_node_list()){
+		fprintf(stderr, "Configured Nodes does not match available memory nodes\n");
+		exit(1);
+	}
+
 	printf("%d nodes available\n", numnodes);
 	fract_nodes = (((numnodes-1)/8)*2) + FRACT_NODES;
 
@@ -566,5 +575,6 @@ int main(int ac, char **av)
 			}
 		}
 	}
+	free(node_to_use);
 	return 0;
 }
